#include "BoodskapMessage.h"
#include <Arduino.h>

BoodskapMessage::BoodskapMessage(unsigned int messageId, JsonBuffer& buffer, const char* domainKey, const char* assetId, const char* apiKey, const char* fwVersion, DeviceModel devModel):
  _domainKey(domainKey), _assetId(assetId), _apiKey(apiKey), _fwVersion(fwVersion), _devModel(devModel), _msg(0), _header(0), _data(0)
{
  _msg = &buffer.createObject();
  _header = &_msg->createNestedObject("header");
  _data = &_msg->createNestedObject("data");

  (*_header)["key"] = _domainKey;
  (*_header)["mid"] = messageId;
  (*_header)["aid"]   = _assetId;
  (*_header)["api"]  = _apiKey;
  (*_header)["dmdl"]  = DeviceModelNames[_devModel];
  (*_header)["fwver"]  = _fwVersion;

}

BoodskapMessage::BoodskapMessage(JsonBuffer& buffer):
  _domainKey(""), _assetId(""), _apiKey(""), _fwVersion(""), _devModel(ARDUINO_GENERIC), _msg(0), _header(0), _data(0)
{
  _msg = &buffer.createObject();
  _header = &_msg->createNestedObject("header");
  _data = &_msg->createNestedObject("data");

}

bool BoodskapMessage::parse(BoodskapMessage& msg, const char* json) {

  DynamicJsonBuffer jsonBuffer;
  JsonObject* root = &jsonBuffer.parseObject(json);

  if (!root->success()) {
    return false;
  }

  if (!root->containsKey("header")) return false;
  if (!root->containsKey("data")) return false;

  JsonObject& h = (*root)["header"];
  JsonObject& d = (*root)["data"];
  
  if (!h.containsKey("key")) return false;
  if (!h.containsKey("mid")) return false;
  if (!h.containsKey("aid")) return false;
  if (!h.containsKey("api")) return false;
  if (!h.containsKey("cmdid")) return false;
  if (!h.containsKey("corrid")) return false;

  msg.header()["key"] = h["key"];
  msg.header()["mid"] = h["mid"];
  msg.header()["aid"] = h["aid"];
  msg.header()["api"] = h["api"];
  msg.header()["cmdid"] = h["cmdid"];
  msg.header()["corrid"] = h["corrid"];

  for (auto dataobj : d) {
    msg.data()[dataobj.key] = d[dataobj.key];
  }

  return true;
}

JsonObject& BoodskapMessage::get() {
  return *_msg;
}

JsonObject& BoodskapMessage::header() {
  return *_header;
}

JsonObject& BoodskapMessage::data() {
  return *_data;
};

void BoodskapMessage::compact() {
  _header->remove("dmdl");
  _header->remove("fwver");
}

void BoodskapMessage::constructAck(bool ack) {
  
  _header->remove("dmdl");
  _header->remove("fwver");
  (*_header)["mid"] = 1;
  
  for (auto dataobj : *_data) {
    _data->remove(dataobj.key);
  }

  (*_data)["ack"] = ack;
}

void BoodskapMessage::clear() {  
  for (auto dataobj : *_data) {
    _data->remove(dataobj.key);
  }
}

JsonObject& BoodskapMessage::set(const char* field, uint8_t val) {
  (*_data)[field]  = val;
  return data();
}

JsonObject& BoodskapMessage::set(const char* field, bool val) {
  (*_data)[field]  = val;
  return data();
}

JsonObject& BoodskapMessage::set(const char* field, int val) {
  (*_data)[field]  = val;
  return data();
}

JsonObject& BoodskapMessage::set(const char* field, uint16_t val) {
  (*_data)[field]  = val;
  return data();
}

JsonObject& BoodskapMessage::set(const char* field, uint32_t val) {
  (*_data)[field]  = val;
  return data();
}

JsonObject& BoodskapMessage::set(const char* field, float val) {
  (*_data)[field]  = val;
  return data();
}

JsonObject& BoodskapMessage::set(const char* field, double val) {
  (*_data)[field]  = val;
  return data();
}

JsonObject& BoodskapMessage::set(const char* field, long val) {
  (*_data)[field]  = val;
  return data();
}

JsonObject& BoodskapMessage::set(const char* field, const char* val) {
  (*_data)[field]  = val;
  return data();
}

JsonObject& BoodskapMessage::set(const char* field, String val) {
  (*_data)[field]  = val;
  return data();
}

